﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RockPaperScissorsSubprogram
{
    class Program
    {
        // Beolvassa a játékos által mutatott figurát.
        static bool readFigure(string message, out string figure)
        {
            Console.WriteLine(message);
            figure = Console.ReadLine();
            if ("ko" == figure || "papir" == figure || "ollo" == figure)
            {
                return true;
            }
            return false;
        }

        // Véletlenszerű értékek közül csak számot tudunk generálni, itt átkonvertáljuk figurává.
        static string intToFigure(int i)
        {
            if (0 == i) return "ko";
            if (1 == i) return "papir";
            if (2 == i) return "ollo";
            return "";
        }

        // A két játékos által mutatott figura alapján eldöntjük, hogy ki nyer.
        // Visszaadott értékek: 0: döntetlen, -1: első játékos nyert, +1: második játékos nyert.
        static int winner(string first, string second)
        {
            if (first == second) return 0;
            if (first == "ko" && second == "ollo"
                ||
                first == "papir" && second == "ko"
                ||
                first == "ollo" && second == "papir")
            {
                return -1;
            }
            return 1;
        }

        // A két játékos által mutatott figura alapján eldöntjük, hogy ki nyer (alternatív verzió).
        // Visszaadott értékek: 0: döntetlen, -1: első játékos nyert, +1: második játékos nyert.
        // Ha a bemenő paraméterek egészek (ehhez szükséges, hogy korábban egésszé konvertáljuk mindkét figurát), akkor valamilyen matematikai képlettel tömörebb kódot tudunk írni.
        // A matematikai képletben most azt használjuk ki, hogy a kő-papír-olló felsorolás a "jó" sorrendben adott, vagyis egy figura a sorrendben közvetlenül előtte állót győzi le, ciklikusan.
        // A ciklukusságot a maradékos osztás ("modulo", a jele % a kódban) művelettel kezeljük.
        static int winner(int first, int second)
        {
            if (first == second) return 0;
            if (first == (second + 1) % 3) return -1;  // kő < papír < olló < kő
            return 1;
        }

        // Függvény, ami visszaadja, hogy a játékos szeretne-e új játékot
        static bool wantNewGame()
        {
            // Rákérdezünk a játékostól, hogy szeretne-e új játékot.
            // Addig olvasunk be szöveget, míg az "i" vagy "n" betű valamelyikét be nem gépelte
            string inputString = "";
            Console.WriteLine();
            Console.WriteLine("Uj jatek? (i/n)");
            do
            {
                inputString = Console.ReadLine();
            } while (!(inputString == "i" || inputString == "n"));  // Csak akkor lépünk tovább, ha "i" vagy "n" betűt ütött be a játékos

            if (inputString == "n")  // ha "n" betűt kaptunk, hamisat adunk vissza, azaz a játékos nem szeretne új játékot
            {
                return false;
            }
            // egyébként igazat adunk vissza (itt már tudjuk, hogy "i"-t kellett kapnunk)
            return true;
        }

        static void Main(string[] args)
        {
            bool newGame = true;        // Addig iterálunk a lenti ciklusban, míg a játékos ki nem lép (newGame == false lesz). Kezdetben a játékos szeretne játszani (newGame == true)
            Random rnd = new Random();  // Új randomszámgenerátor létrehozása. A Next(...) hívás ad vissza egy következő véletlen számot
            while (newGame)
            {
                Console.Clear();        // Minden játék elején letöröljük a konzol ablakot

                // Játékos által mutatott érték beolvasása
                string playerFigureString;
                while (!readFigure("Mit mutatsz?", out playerFigureString))
                {
                    Console.WriteLine("Ervenyes figurak: " + "ko" + ", " + "papir" + ", " + "ollo");
                }

                // Gépi játékos által mutatott figura generálása
                int compFigure = rnd.Next(0, 3);  // Véletlen szám generálása (0: kő, 1: papír, 2: olló). Next(min, max), min és max-1 között ad vissza értékeket, tehát max nincs benne
                string compFigureString = intToFigure(compFigure);
                Console.WriteLine("A gep ezt mutatta: " + compFigureString);


                // Megnézzük, hogy ki nyert, és annak megfelelően kiírjuk az eredményt.
                int result = winner(playerFigureString, compFigureString);
                switch (result)
                {
                    case -1:
                        Console.WriteLine("Gratulalunk, nyertel!");
                        break;
                    case 0:
                        Console.WriteLine("Dontetlen!");
                        break;
                    case 1:
                        Console.WriteLine("Sajnos a gep nyert :-(");
                        break;
                    default:
                        break;
                }

                // Alprogramban bekérjük a játékos választását, a visszatérési érték mondja meg, hogy szeretne-e új játékot.
                newGame = wantNewGame();
            }
        }
    }

}
